@@ -34,6 +34,7 @@ struct TempMBR {
    struct MBRRecord partitions[4];
    uint16_t MBRSignature;
 }; // struct TempMBR
+#pragma pack ()
 
 // Possible states of the MBR
 enum MBRValidity {invalid, gpt, hybrid, mbr};
@@ -62,6 +63,7 @@ protected:
 public:
    BasicMBRData(void);
    BasicMBRData(string deviceFilename);
+   BasicMBRData(const BasicMBRData &);
    ~BasicMBRData(void);
    BasicMBRData & operator=(const BasicMBRData & orig);
 
