@@ -57,6 +57,7 @@ struct GPTHeader {
    uint32_t partitionEntriesCRC;
    unsigned char reserved2[GPT_RESERVED];
 }; // struct GPTHeader
+#pragma pack ()
 
 // Data in GPT format
 class GPTData {
@@ -91,6 +92,7 @@ protected:
 public:
    // Basic necessary functions....
    GPTData(void);
+   GPTData(const GPTData &);
    GPTData(string deviceFilename);
    virtual ~GPTData(void);
    GPTData & operator=(const GPTData & orig);
